(*
////////////////////////////////////////////////////////////////////////////////
// DBFLTCOM.PAS : DATABASE LOOKUP COMBOBOX WITH FILTER                        //
// Copyright  1996-1999 Steve Flynn. All Rights Reserved                     //
//----------------------------------------------------------------------------//
// By using this source code you agree to be bound by the terms of the        //
// following Software License Agreement. Any breach of this agreement will be //
// prosecuted to the full extent of the law.                                  //
////////////////////////////////////////////////////////////////////////////////

LICENSE AGREEMENT - MARCH 1999

USE OF THE SOFTWARE

Steven M. Flynn (the "Author") grants you, the end user, a non-exclusive
single-user license to use the supplied software program and all associated
materials including the Source Code (the "Software"). The Software and all
Intellectual Property embodied in the Software always remains the property of
the Author. Your use of the Software indicates your acceptance in full of the
terms and conditions of this agreement.


COPYING THE SOFTWARE

You may freely copy the Software provided that such copies do not infringe upon
any other part of this agreement.


DISTRIBUTION OF THE SOFTWARE

The Software may be re-distributed royalty free providing that it is not
modified in any way and that no charge is made for the Software itself. Only a
nominal fee for media and/or copying services is allowed. All files, including
the License Agreement, that are supplied by the Author with the Software must
also be included in any such distribution.


DERIVED WORKS

You may NOT use the Software to derive any works which are based substantially
in part or in full on the Software or any portion of the Software. Any such
derived works will remain the property of the Author.


MODIFIED VERSIONS OF THE SOFTWARE

All updated, revised, modified or improved versions of the Software are also
subject to the conditions of this license agreement and shall remain the
property of the Author. You are required under the terms of this agreement to
submit to the Author any and all improvements or additions made by you to the 
Software for future inclusion in the general distribution of the Software by
the Author.


TERMINATION OF THIS AGREEMENT

You may terminate this License Agreement at any time by destroying all copies
of the Software in your possession.


LIMITED WARRANTY

The Software is distributed and licensed "AS IS". The Author specifically
disclaims all other warranties, express or implied, including but not limited
to, implied warranties of merchantability and fitness for a particular purpose,
with regard to the Software.


DAMAGES

By using the Software you do so at your own risk. In no event shall the Author
be responsible for any damages whatsoever (including but not limited to,
damages for loss of business profits, business interruption, loss of business
information, or any other pecuniary loss or any other real or consequential 
damages) arising out of the use or inability to use this product.


LIABILITY

In the event of failure of the Software, for any reason, the Author's sole
liability shall be to refund to you the amount paid by you to the Author for
the use of the Software.


COPYRIGHT

The Software and the Intellectual Property embodied in the Software shall
always remain the property of the Author and is protected by Australian
Copyright Law and International Treaty Provisions.


AGREEMENT

Your use of the Software indicates your agreement with the above terms and
conditions. If you do not agree with these terms and conditions then you must
destroy all copies of the software in your possession.


CONSUMER RIGHTS

If any clause in the above Software License Agreement is in violation of your
rights under your local laws then you must cease using the Software
immediately. It is your own responsibility to be aware of your legal rights
under your own local laws.


END OF SOFTWARE LICENSE AGREEMENT

*)

{$A-,B-,D-,F-,G+,I-,L-,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y-,Z-}
{$IFDEF WIN32}
{$H+,J+}
{$M 16384,1048576}
{$IMAGEBASE $00400000}
{$R DBFLTCOM.R32}
{$ELSE}
{$K-}
{$M 16384,8192}
{$R DBFLTCOM.R16}
{$ENDIF}
{$C PRELOAD}

unit DBFltCom;

interface

uses
{$IFNDEF WIN32}
	Menus, DBITypes, DBIProcs,
{$ELSE}
	BDE,
{$ENDIF}
{$IFDEF VER100}
	DBCtrls,
{$ENDIF}
{$IFDEF VER120}
	DBCtrls,
{$ENDIF}
	WinProcs, Graphics, Dialogs, Classes, Controls, Messages, Buttons, StdCtrls,
	DB, DBTables, Grids;

const
	WM_LISTCLICK = WM_USER + 77;

type
{// Miscellaneous /////////////////////////////////////////////////////////////}
	TDBFilterCombo = class;
	TDBFilterComboStyle = (csDropDown, csDropDownList);
	TNewLookUpRecord = procedure(Sender: TObject; var Cancelled: Boolean) of object;
	TOnListClick = procedure (Combo: TDBFilterCombo; ListBox: TStringGrid; Index: integer) of Object;
	TDropOption = (doVertLine, doHorzLine, doFixedVertLine, doFixedHorzLine);
	TDropOptions = set of TDropOption;
	TConvertCase = (ccAsTyped, ccUpperCase, ccLowerCase);
	TAutoDrop = (adNormal, adOnChange, adOnFocus);

{// TDBFilterButton ///////////////////////////////////////////////////////////}
	TDBFilterButton = class(TSpeedButton)
	private
		FCombo: TDBFilterCombo;

	protected
		procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
	end;


{// TDBFilterList /////////////////////////////////////////////////////////////}
	TDBFilterList = class(TStringGrid)
	private
		FCombo: TDBFilterCombo;
		FLooseClick, FSetScrollBars: boolean;
		FLookupItems: TStrings;
		FLookupCurs: TList;
		procedure SetRow(Value: integer);
		procedure SetTopRow(Value: integer);
		function GetRow: integer;
		function GetTopRow: integer;

	protected
		procedure CreateWnd; override;
		procedure CreateParams(var Params: TCreateParams); override;
		procedure Click; override;
		procedure Paint; override;

	public
		property TopRow: integer read GetTopRow write SetTopRow;
		property Row: integer read GetRow write SetRow;
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
	end;


{// TDBFilterCombo ////////////////////////////////////////////////////////////}
	TDBFilterCombo = class(TCustomEdit)
	private
		FDataLink: TFieldDataLink;
		FDropGrid: TDBFilterList;
		FButtonHolder: TWinControl;
		FButton: TDBFilterButton;
		FLookupSource: TDataSource;
		FLookupFld, FFilterFld: TField;
		FLookupSrc: TDataSet;
		FLookupItem, FDisplayField, FLookupField: string;
		FFilterField, FFilterMin, FFilterMax: string;
		FCanvas: TControlCanvas;
		FDropWidth, FDropDownCount, FDropWid: integer;
		FLookupUseCase, FIncSearch, FIncUseCase, FDataBaseOK: boolean;
		FDropOpen, FIncErrorBeep, FUpdating, FFilterExclusive: boolean;
		FFilterRelaxed, FNoFilter, FParentDropFont, FCallingEvent: boolean;
		FAutoDrop: TAutoDrop;
		FCase: TConvertCase;
		FStyle: TDBFilterComboStyle;
		FDropNames, FCurrentText: string;
		FDisplayList: TStringList;
		FDropFixedCol, FDropColor: TColor;
		FDropFont: TFont;
		FColNames, FDropLineWidth: integer;
		FDropOptions: TDropOptions;
		FDropScroll: TScrollStyle;
		FOnListClick: TOnListClick;
		FOnLookupChanged: TOnListClick;
		FOnNewLookupRecord: TNewLookUpRecord;
		FOnBeforeLookup, FOnAfterLookup: TNotifyEvent;
		function GetDataSource: TDataSource;
		function GetDataField: string;
		function FitsFilter: boolean;
		function LookupExists: boolean;
		procedure SetDataSource(Src: TDataSource);
		procedure SetDataField(Str: string);
		procedure SetLookupSource(Src: TDataSource);
		procedure SetLookupField(Str: string);
		procedure SetDisplayField(Str: string);
		procedure SetFilterField(Str: string);
		procedure SetIncSearch(Value: boolean);
		procedure SetStyle(Value: TDBFilterComboStyle);
		procedure SetDropFont(Value: TFont);
		procedure SetDropWidth(Value: integer);
		procedure SetDropLineWidth(Value: integer);
		procedure DataLinkUpdate;
		procedure NonEditMouseDown(var Message: TWMLButtonDown);
		procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
		procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
		procedure WMSize(var Message: TWMSize); message WM_SIZE;
		procedure WMMove(var Message: TWMMove); message WM_MOVE;
		procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
		procedure WMNCPaint(var Message: TWMNCPaint); message WM_NCPAINT;
		procedure WMGetText(var Message: TWMGetText); message WM_GETTEXT;
		procedure WMSetText(var Message: TWMSetText); message WM_SETTEXT;
		procedure WMPaste(var Message: TMessage); message WM_PASTE;
		procedure WMCut(var Message: TMessage); message WM_CUT;
		procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
		procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
		procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
		procedure WMListClick(var Message: TMessage); message WM_LISTCLICK;
		procedure WMKeyDown(var Message: TWMKey); message WM_KEYDOWN;
		procedure WMKeyPress(var Message:TWMKey); message WM_CHAR;
		procedure WMKeyUp(var Message: TWMKey); message WM_KEYUP;
		procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
		procedure CMCancelMode(var Message: TCMCancelMode); message CM_CANCELMODE;
		procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
		procedure AssignFields;
(*WINDOWS BUG		procedure SetEditRect;*)
		procedure CloseUp;
		Procedure ReDrop;
		procedure DataChange(Sender: TObject);
		procedure DataUpdate(Sender: TObject);
		procedure LookupGridRow(Row: integer; OldLookup: string);

	protected
		procedure Notification(AComponent: TComponent; Operation: TOperation); override;
		procedure CreateParams(var Params: TCreateParams); override;
		procedure Change; override;
		procedure Loaded; override;
		procedure DoExit; override;
		procedure DropDown(Show: boolean);

	public
		class function GetCopyright: string;
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;

	published
		property ConvertCase: TConvertCase read FCase write FCase default ccAsTyped;
		property DropDownColumnNames: string read FDropNames write FDropNames;
		property DropDownCount: integer read FDropDownCount write FDropDownCount default 8;
		property DropDownColor: TColor read FDropColor write FDropColor default clWindow;
		property DropDownFixedColor: TColor read FDropFixedCol write FDropFixedCol default clBtnFace;
		property DropDownFont: TFont read FDropFont write SetDropFont;
		property DropDownLineWidth: integer read FDropLineWidth write SetDropLineWidth default 1;
		property DropDownOpen: boolean read FDropOpen;
		property DropDownOptions: TDropOptions read FDropOptions write FDropOptions default [];
		property DropDownScrollBars: TScrollStyle read FDropScroll write FDropScroll default ssVertical;
		property DropDownWidth: integer read FDropWid write SetDropWidth default 0;
		property DataSource: TDataSource read GetDataSource write SetDataSource;
		property DataField: string read GetDataField write SetDataField;
		property IncrementalAutoDropDown: TAutoDrop read FAutoDrop write FAutoDrop default adNormal;
		property IncrementalErrorBeep: boolean read FIncErrorBeep write FIncErrorBeep default True;
		property IncrementalSearch: boolean read FIncSearch write SetIncSearch default False;
		property IncrementalUseCase: boolean read FIncUseCase write FIncUseCase default False;
		property LookupSource: TDataSource read FLookupSource write SetLookupSource;
		property LookupDisplay: string read FDisplayField write SetDisplayField;
		property LookupField: string read FLookupField write SetLookupField;
		property LookupFilterUseCase: boolean read FLookupUseCase write FLookupUseCase default False;
		property LookupFilterField: string read FFilterField write SetFilterField;
		property LookupFilterMin: string read FFilterMin write FFilterMin;
		property LookupFilterMax: string read FFilterMax write FFilterMax;
		property LookupFilterExclusive: boolean read FFilterExclusive write FFilterExclusive default False;
		property LookupFilterRelaxed: boolean read FFilterRelaxed write FFilterRelaxed default False;
		property Style: TDBFilterComboStyle read FStyle write SetStyle default csDropDown;
		property ParentDropDownFont: boolean read FParentDropFont write FParentDropFont default True;
		property AutoSelect;
		property Color;
		property Ctl3D;
		property DragCursor;
		property DragMode;
		property Enabled;
		property Font;
		property MaxLength;
		property ParentColor;
		property ParentCtl3D;
		property ParentFont;
		property ParentShowHint;
		property PopupMenu;
		property ShowHint;
		property TabOrder;
		property TabStop;
		property Visible;
		property OnChange;
		property OnClick;
		property OnDblClick;
		property OnDragDrop;
		property OnDragOver;
		property OnEndDrag;
		property OnEnter;
		property OnExit;
		property OnKeyDown;
		property OnKeyPress;
		property OnKeyUp;
		property OnMouseDown;
		property OnMouseMove;
		property OnMouseUp;
		property OnListClick: TOnListClick read FOnListClick write FOnListClick;
		property OnNewLookupRecord: TNewLookUpRecord read FOnNewLookupRecord write FOnNewLookupRecord;
		property OnAfterLookup: TNotifyEvent read FOnAfterLookup write FOnAfterLookup;
		property OnBeforeLookup: TNotifyEvent read FOnBeforeLookup write FOnBeforeLookup;
		property OnLookupChanged: TOnListClick read FOnLookupChanged write FOnLookupChanged;
	end;


{// Registration //////////////////////////////////////////////////////////////}
procedure Register;

implementation

uses
{$IFNDEF WIN32}
	WinTypes,
{$ENDIF}
	DBConsts, SysUtils, Forms,
{$IFDEF VER100}
	Consts;
{$ELSE}
{$IFDEF VER120}
	Consts;
{$ELSE}
	LibConst;
{$ENDIF}
{$ENDIF}

var
	ButtonSize: integer;
	Tricky: string;
	CopyrightNotice: string;

{$IFNDEF VER90}
type
	TRecNoStatus = (rnDbase, rnParadox, rnNotSupported);
{$ENDIF}


{// TDBFilterButton ///////////////////////////////////////////////////////////}
constructor TDBFilterButton.Create(AOwner: TComponent);
	begin
	inherited Create(AOwner);
	Width := ButtonSize;
	Height := ButtonSize;
	FCombo := TDBFilterCombo(AOwner);
	Parent := FCombo.FButtonHolder;
	Glyph.Handle := LoadBitmap(0, PChar(32738));
	Visible := True;
	end;


destructor TDBFilterButton.Destroy;
	begin
	DeleteObject(Glyph.Handle);
	inherited Destroy;
	end;


procedure TDBFilterButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
	var
	r1, r2: TRect;
	begin
	FCombo.SetFocus;
	inherited MouseDown(Button, Shift, X, Y);
	if FCombo.FDropGrid <> nil then
		with FCombo do
			begin
			if IsWindowVisible(FDropGrid.Handle) then
				begin
				GetClipBox(FDropGrid.Canvas.Handle, r1);
				r2 := FDropGrid.GetClientRect;
				OffsetRect(r1, -r1.Left, -r1.Top);
				OffsetRect(r2, -r2.Left, -r2.Top);
				if not EqualRect(r1, r2) then
					DropDown(True)
				else
					CloseUp;
				end
			else
				DropDown(True);
			end;
	end;


{// TDBFilterList /////////////////////////////////////////////////////////////}
constructor TDBFilterList.Create(AOwner: TComponent);
	begin
	FCombo := TDBFilterCombo(AOwner);
	inherited Create(AOwner);
	Visible := False;
	FLooseClick := False;
	FSetScrollBars := True;
	Ctl3D := False;
	Parent := GetParentForm(FCombo);
	ParentColor := False;
	ParentCtl3D := False;
	ParentFont := False;
	ParentShowHint := False;
	TabStop := False;
	FixedCols := 0;
	FixedRows := 0;
	ShowHint := False;
	Color := FCombo.FDropColor;
	FixedColor := FCombo.FDropFixedCol;
	ScrollBars := FCombo.FDropScroll;
	Options := [goRowSelect, goThumbTracking];
	if doVertLine in FCombo.FDropOptions then
		Options := Options + [goVertLine];
	if doHorzLine in FCombo.FDropOptions then
		Options := Options + [goHorzLine];
	if doFixedVertLine in FCombo.FDropOptions then
		Options := Options + [goFixedVertLine];
	if doFixedHorzLine in FCombo.FDropOptions then
		Options := Options + [goFixedHorzLine];
	if FCombo.FParentDropFont then
		Font := FCombo.Font
	else
		Font := FCombo.FDropFont;
	if FCombo.FDropLineWidth > 0 then
		GridLineWidth := FCombo.FDropLineWidth;
	DefaultRowHeight := Abs(Font.Height) + Abs(Font.Height) div 2;
	DefaultColWidth := 1;
	FLookupItems := TStringList.Create;
	FLookupCurs := TList.Create;
	end;


destructor TDBFilterList.Destroy;
	begin
	FLookupItems.Free;
	FLookupCurs.Free;
	inherited Destroy;
	end;


procedure TDBFilterList.CreateWnd;
	begin
	inherited CreateWnd;
	WinProcs.SetParent(Handle, GetDesktopWindow);
	end;


procedure TDBFilterList.CreateParams(var Params: TCreateParams);
	begin
	inherited CreateParams(Params);
	Params.WindowClass.Style := Params.WindowClass.Style + CS_SAVEBITS;
{$IFDEF WIN32}
	Params.ExStyle := Params.ExStyle + WS_EX_TOOLWINDOW;
{$ENDIF}
	end;


procedure TDBFilterList.SetRow(Value: integer);
	begin
	FLooseClick := True;
	inherited Row := Value;
	end;


procedure TDBFilterList.SetTopRow(Value: integer);
	begin
	FLooseClick := True;
	inherited TopRow := Value;
	end;


function TDBFilterList.GetRow;
	begin
	Result := inherited Row;
	end;


function TDBFilterList.GetTopRow;
	begin
	Result := inherited TopRow;
	end;


procedure TDBFilterList.Click;
	begin
	inherited Click;
	if not FLooseClick then
		PostMessage(FCombo.Handle, WM_LISTCLICK, 1, 0)
	else
		FLooseClick := False;
	end;


procedure TDBFilterList.Paint;
	begin
	if FSetScrollBars then
		begin
		if not (ScrollBars in [ssVertical, ssBoth]) then
			SetScrollRange(Handle, SB_VERT, 0, 0, False);
		if not (ScrollBars in [ssHorizontal, ssBoth]) then
			SetScrollRange(Handle, SB_HORZ, 0, 0, False);
		FSetScrollBars := False;
		end;
	inherited Paint;
	end;


{// TDBFilterCombo ////////////////////////////////////////////////////////////}
constructor TDBFilterCombo.Create(AOwner: TComponent);
	begin
	inherited Create(AOwner);
{$IFDEF WIN32}
	Width := 145;
	Height := ButtonSize + 5;
{$ENDIF}
	AutoSize := False;
	ReadOnly := False;
	FDataLink := TFieldDataLink.Create;
	FDataLink.Control := self;
	FDataLink.OnDataChange := DataChange;
	FDataLink.OnUpdateData := DataUpdate;
	FStyle := csDropDown;
	FFilterExclusive := False;
	FDropDownCount := 8;
	FDropWidth := 0;
	FDropWid := 0;
	FDisplayField := '';
	FLookupField := '';
	FFilterField := '';
	FFilterMin := '';
	FFilterMax := '';
	FLookupItem := '';
	FCase := ccAsTyped;
	FDataBaseOK := False;
	FNoFilter := False;
	FAutoDrop := adNormal;
	FCallingEvent := False;
	FParentDropFont := True;
	FIncErrorBeep := True;
	FButtonHolder := TWinControl.Create(self);
	FButtonHolder.Parent := self;
	FButton := TDBFilterButton.Create(self);
	FIncSearch := False;
	FIncUseCase := False;
	FLookupUseCase := False;
	FDropOpen := False;
	FUpdating := False;
	FColNames := 0;
	FCurrentText := '';
	FDropNames := '';
	FDropColor := clWindow;
	FDropFixedCol := clBtnFace;
	FDropLineWidth := 1;
	FDropOptions := [];
	FDropScroll := ssVertical;
	FDropFont := TFont.Create;
	FDisplayList := TStringList.Create;
	end;


destructor TDBFilterCombo.Destroy;
	begin
	CloseUp;
	FDataBaseOK := False;
	FDataLink.OnDataChange := nil;
	FDataLink.OnUpdateData := nil;
	FDisplayList.Free;
	FDropFont.Free;
	FDataLink.Free;
	FDataLink := nil;
	inherited Destroy;
	end;


class function TDBFilterCombo.GetCopyright: string;
	begin
	Result := CopyrightNotice;
	end;


procedure TDBFilterCombo.CreateParams(var Params: TCreateParams);
	begin
	inherited CreateParams(Params);
	Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
	end;


procedure TDBFilterCombo.Loaded;
	begin
	inherited Loaded;
	AssignFields;
	end;


procedure TDBFilterCombo.AssignFields;
	var
	i, j: integer;
	OldOK, OldFilt: boolean;
	begin
	OldOK := FDataBaseOK;
	OldFilt := FNoFilter;
	FDataBaseOK := False;
	FNoFilter := False;
	if (csLoading in ComponentState)
		or (csDestroying in ComponentState) then
		exit;
	FLookupSrc := nil;
	FLookupFld := nil;
	FFilterFld := nil;
	if (FLookupSource <> nil) then
		FLookupSrc := FLookupSource.Dataset;
	if (FLookupSrc <> nil) and (FLookupField <> '') then
		FLookupFld := FLookupSrc.FindField(FLookupField);
	if (FLookupSrc <> nil) and (FFilterField <> '') then
		FFilterFld := FLookupSrc.FindField(FFilterField);
	j := 0;
	if FLookupSrc <> nil then
		for i := 0 to FDisplayList.Count - 1 do
			begin
			FDisplayList.Objects[i] := FLookupSrc.FindField(FDisplayList[i]);
			if FDisplayList.Objects[i] <> nil then
				Inc(j);
			end;
	if (j = FDisplayList.Count)
		and (FLookupSrc <> nil)
		and (FLookupFld <> nil)
		and (FDataLink.Datasource <> nil)
		and (FDataLink.FieldName <> '')
		and FLookupSrc.Active
		and FDataLink.Active then
		begin
		if (FFilterFld = nil) or (FFilterMax = '') or (FFilterMin = '') then
			FNoFilter := True;
		FDataBaseOK := True;
		if (not OldOK) or ((OldFilt <> FNoFilter) and FDataBaseOK) then
			DataChange(self);
		end;
	end;


procedure TDBFilterCombo.CMCancelMode(var Message: TCMCancelMode);
	begin
	with Message do
		if (Sender <> Self)
			and (Sender <> FButtonHolder)
			and (Sender <> FButton)
			and (Sender <> FDropGrid) then
			CloseUp;
	end;


procedure TDBFilterCombo.CMEnabledChanged(var Message: TMessage);
	begin
	inherited;
	FButton.Enabled := Enabled;
	FButtonHolder.Repaint;
	FButton.Repaint;
	end;


procedure TDBFilterCombo.WMEraseBkgnd(var Message: TWMEraseBkgnd);
	begin
	inherited;
	FButtonHolder.Refresh;
	FButton.Refresh;
	end;


procedure TDBFilterCombo.WMLButtonDown(var Message: TWMLButtonDown);
	begin
	if ReadOnly then
		NonEditMouseDown(Message)
	else
		inherited;
	end;


procedure TDBFilterCombo.WMLButtonUp(var Message: TWMLButtonUp);
	begin
	if ReadOnly then
		MouseCapture := False;
	inherited;
	end;


procedure TDBFilterCombo.WMLButtonDblClk(var Message: TWMLButtonDblClk);
	begin
	if ReadOnly then
		NonEditMouseDown(Message)
	else
		inherited;
	end;


procedure TDBFilterCombo.NonEditMouseDown(var Message: TWMLButtonDown);
	var
	CtrlState: TControlState;
	begin
	SetFocus;
	HideCaret(Handle);
	if (FDropGrid <> nil) and IsWindowVisible(FDropGrid.Handle) then
		CloseUp
	else
		DropDown(True);
	MouseCapture := True;
	if csClickEvents in ControlStyle then
		begin
		CtrlState := ControlState;
		Include(CtrlState, csClicked);
		ControlState := CtrlState;
		end;
	with Message do
		MouseDown(mbLeft, KeysToShiftState(Keys), XPos, YPos);
	end;


procedure TDBFilterCombo.CloseUp;
	begin
	if FDropGrid = nil then
		exit;
	SetWindowPos(FDropGrid.Handle, 0, -2, -2, 1, 1, SWP_HIDEWINDOW + SWP_NOACTIVATE);
	FDropOpen := IsWindowVisible(FDropGrid.Handle);
	end;


procedure TDBFilterCombo.DropDown(Show: boolean);
	var
	i, j, tw, dropw, XBox, YBox, HBox, HLine, VLine: integer;
	showmode: integer;
	begin
	if FDropGrid = nil then
		exit;
	if FParentDropFont then
		FDropGrid.Font := Font
	else
		FDropGrid.Font := FDropFont;
	FDropGrid.Canvas.Font := FDropGrid.Font;
	FDropGrid.GridLineWidth := FDropLineWidth;
	FDropGrid.DefaultRowHeight := Abs(FDropGrid.Font.Height) + Abs(FDropGrid.Font.Height) div 2;
	HLine := FDropGrid.GridLineWidth * Ord((doHorzline in FDropOptions) or (doFixedHorzline in FDropOptions));
	if FDropGrid.RowCount = 0 then
		HBox := FDropGrid.DefaultRowHeight
	else
		if FDropGrid.RowCount < FDropDownCount then
			HBox := FDropGrid.RowCount * (FDropGrid.DefaultRowHeight + HLine) + {$IFDEF WIN32} 4 {$ELSE} 2 {$ENDIF}
		else
			HBox := FDropDownCount * (FDropGrid.DefaultRowHeight + HLine) + {$IFDEF WIN32} 4 {$ELSE} 2 {$ENDIF};
	VLine := FDropLineWidth * Ord((doVertLine in FDropOptions) or (doFixedVertLine in FDropOptions));
	for i := 0 to FDisplayList.Count - 1 do
		FDropGrid.ColWidths[i] := 4;
	for j := 0 to FDropGrid.RowCount - 1 do
		for i := 0 to FDisplayList.Count - 1 do
			begin
			tw := FDropGrid.Canvas.TextWidth(FDropGrid.Cells[i, j]) + 4;
			if tw > FDropGrid.ColWidths[i] then
				FDropGrid.ColWidths[i] := tw;
			end;
	for i := 0 to FDisplayList.Count - 1 do
		if FDropGrid.ColWidths[i] = 4 then
			FDropGrid.ColWidths[i] := (Width div FDisplayList.Count) - VLine - 1;
	dropw := 0;
	for i := 0 to FDisplayList.Count - 1 do
		Inc(dropw, FDropGrid.ColWidths[i] + VLine);
	if (FDropGrid.RowCount > FDropDownCount)
		and (FDropScroll in [ssVertical, ssBoth]) then
		Inc(dropw, GetSystemMetrics(SM_CXVSCROLL));
	Inc(dropw, 4);
	if FDropWid = 0 then
		begin
		FDropWidth := Width - VLine;
		if FDisplayList.Count > 1 then
			FDropWidth := dropw;
		if (FDropWidth < dropw) and (FDropScroll in [ssHorizontal, ssBoth]) then
			Inc(HBox, GetSystemMetrics(SM_CYHSCROLL));
{$IFNDEF WIN32}
		Dec(FDropWidth, 3);
{$ENDIF}
		end
	else
		FDropWidth := FDropWid;
	if FDisplayList.Count = 1 then
		FDropGrid.ColWidths[0] := FDropWidth;
	YBox := ClientOrigin.Y + Height - 2;
	if (YBox + HBox > Screen.Height) then
		YBox := ClientOrigin.Y - HBox {$IFDEF WIN32} - 3 {$ENDIF};
	if YBox < 0 then
		YBox := 0;
	XBox := ClientOrigin.X + Width - FDropWidth {$IFDEF WIN32} - 2 {$ENDIF};
	if (XBox + FDropWidth > Screen.Width) then
		XBox := Screen.Width - FDropWidth;
	if (XBox < 0) then
		XBox := 0;
	if Show then
		showmode := SWP_SHOWWINDOW + SWP_NOACTIVATE
	else
		showmode := SWP_HIDEWINDOW + SWP_NOACTIVATE;
	SetWindowPos(FDropGrid.Handle, HWND_TOP, -FDropWidth - 1, -Hbox - 1, FDropWidth, HBox, showmode);
	SetWindowPos(FDropGrid.Handle, HWND_TOP, XBox, YBox, FDropWidth, HBox, showmode);
	FDropGrid.FLooseClick := False;
	FDropOpen := IsWindowVisible(FDropGrid.Handle);
	if FDropOpen
		and (Text = '')
		and (FDropGrid.RowCount > 0)
		and (FDropGrid.Row > -1)
		and (FDropGrid.Cells[0, FDropGrid.Row] <> '') then
		LookupGridRow(FDropGrid.Row, '');
	end;


procedure TDBFilterCombo.WMSetFocus(var Message: TWMSetFocus);
	var
	i, j, GridRow: integer;
	s, t: string;
	CursorProps: CurProps;
    FRecProps: RecProps;
	FRecNoStatus: TRecNoStatus;
	begin
	inherited;
	if FCallingEvent then
		if (FDropGrid <> nil) and IsWindowVisible(FDropGrid.Handle) then
			DropDown(True)
		else
			exit;
	DbiGetCursorProps(
{$IFDEF VER100}
		TBDEDataSet
{$ENDIF}
{$IFDEF VER120}
		TBDEDataSet
{$ENDIF}
		(FLookupSrc).Handle, CursorProps);
	FRecNoStatus := TRecNoStatus(CursorProps.ISeqNums);
	if AutoSelect then
		SelectAll;
	if FDropGrid = nil then
		begin
		GridRow := -1;
		if FDropNames = '' then
			FColNames := 0
		else
			FColNames := 1;
		FDropGrid := TDBFilterList.Create(self);
		FDropGrid.Enabled := False;
		FDropGrid.ColCount := FDisplayList.Count;
		FDropGrid.RowCount := 0;
		if Assigned(FOnBeforeLookup) then
			begin
			FCallingEvent := True;
			FOnBeforeLookup(Self);
			FCallingEvent := False;
			end;
		if FColNames = 1 then
			begin
			Inc(GridRow);
			FDropGrid.RowCount := GridRow + 1;
			FDropGrid.FLookupItems.Add(#1); 		{dummy for header row}
			FDropGrid.FLookupCurs.Add(Pointer(-1)); {dummy for header row}
			s := FDropNames;
			for j := 0 to FDisplayList.Count - 1 do
				begin
				i := Pos(';', s);
				if i > 1 then
					begin
					t := Copy(s, 1, i - 1);
					Delete(s, 1, i);
					end
				else
					begin
					t := s;
					s := '';
					end;
				if length(t) > 0 then
					FDropGrid.Cells[j, 0] := t;
				end;
			end;
		if FDataBaseOK then
			begin
			FLookupSrc.Cancel;
			FLookupSrc.First;
			while not FLookupSrc.EOF do
				begin
				if (FNoFilter or FitsFilter) then
					begin
					Inc(GridRow);
					FDropGrid.RowCount := GridRow + 1;
					for i := 0 to FDisplayList.Count - 1 do
						FDropGrid.Cells[i, GridRow] := TField(FDisplayList.Objects[i]).Text;
					FDropGrid.FLookupItems.Add(FLookupFld.Text);
					DbiGetRecord(
{$IFDEF VER100}
						TBDEDataSet
{$ENDIF}
{$IFDEF VER120}
						TBDEDataSet
{$ENDIF}
						(FLookupSrc).Handle, dbiNoLock, nil, @FRecProps);
					case FRecNoStatus of
						rnParadox:
							FDropGrid.FLookupCurs.Add(Pointer(FRecProps.iSeqNum));
						rnDBase:
							FDropGrid.FLookupCurs.Add(Pointer(FRecProps.iPhyRecNum));
						else
							FDropGrid.FLookupCurs.Add(Pointer(-1));
						end;
					FDropGrid.Enabled := True;
					end;
				FLookupSrc.Next;
				end;
			end;
		if Assigned(FOnAfterLookup) then
			begin
			FCallingEvent := True;
			FOnAfterLookup(Self);
			FCallingEvent := False;
			end;
		if (FColNames = 1) and (FDropGrid.RowCount = 1) then {empty list with header}
			FDropGrid.RowCount := 2;
		FDropGrid.FixedRows := FColNames;
		if FDropGrid.Enabled then
			for i := 0 to FDropGrid.RowCount - 1 do
				if FLookupItem = FDropGrid.FLookupItems[i] then
					begin
					FLookupSrc.First;
					FLookupSrc.MoveBy(integer(FDropGrid.FLookupCurs[i]));
					FDropGrid.TopRow := i;
					FDropGrid.Row := i;
					break;
					end;
		end;
	if (FDropGrid <> nil) and (FAutoDrop = adOnFocus) then
		DropDown(True);
	if ReadOnly then
		begin
		HideCaret(Handle);
		Invalidate;
		end;
	FButton.Repaint;
	end;


procedure TDBFilterCombo.WMKillFocus(var Message: TWMKillFocus);
	begin
	inherited;
	if FCallingEvent then
		exit;
	if (FDropGrid <> nil) and (FStyle = csDropDownList) then
		if (FDropGrid.RowCount > 0) and (FDropGrid.Row > -1) then
			Text := FDropGrid.Cells[0, FDropGrid.Row]
		else
			Text := '';
	CloseUp;
	FDropGrid.Free;
	FDropGrid := nil;
	if ReadOnly then
		Invalidate;
	FButton.Repaint;
	end;


procedure TDBFilterCombo.WMMove(var Message: TWMMove);
	begin
	inherited;
	ReDrop;
	end;


procedure TDBFilterCombo.WMSize(var Message: TWMSize);
	var
	No3D, MinHeight: integer;
	function GetMinHeight: Integer;
		var
		DC: HDC;
		SaveFont: HFont;
		i: integer;
		SysMetrics, Metrics: TTextMetric;
		begin
		DC := GetDC(0);
		GetTextMetrics(DC, SysMetrics);
		SaveFont := SelectObject(DC, Font.Handle);
		GetTextMetrics(DC, Metrics);
		SelectObject(DC, SaveFont);
		ReleaseDC(0, DC);
		i := SysMetrics.tmHeight;
		if i > Metrics.tmHeight then
			i := Metrics.tmHeight;
		i := i div 4;
		Result := Metrics.tmHeight + i + GetSystemMetrics(SM_CYBORDER) * 4 + 1;
		end;
	begin
	if Ctl3D then
		No3D := 0
	else
		No3D := 2;
	inherited;
	MinHeight := GetMinHeight;
	if Height < MinHeight then
		Height := MinHeight;
	if not NewStyleControls then
		FButtonHolder.SetBounds(ClientWidth - ButtonSize, 0, ButtonSize, ClientHeight)
	else
{$IFDEF WIN32}
		FButtonHolder.SetBounds(ClientWidth - ButtonSize - No3D, 0 + No3D, ButtonSize, ClientHeight - No3D * 2);
{$ELSE}
		FButtonHolder.SetBounds(ClientWidth - ButtonSize - 2, 2, ButtonSize, ClientHeight - 4);
{$ENDIF}
	FButton.Height := FButtonHolder.Height;
(*WINDOWS BUG	SetEditRect;*)
	ReDrop;
	end;


procedure TDBFilterCombo.WMGetText(var Message: TWMGetText);
	begin
	if not FDataBaseOK then
		AssignFields;
	inherited;
	end;


procedure TDBFilterCombo.WMSetText(var Message: TWMSetText);
	begin
	if not FDataBaseOK then
		AssignFields;
	inherited;
	end;


procedure TDBFilterCombo.WMNCPaint(var Message: TWMNCPaint);
	begin
	if Length(Text) = 0 then
		DataChange(self);
	inherited;
	ReDrop;
	end;


procedure TDBFilterCombo.ReDrop;
	var
	WasDropped: boolean;
	begin
	if FDropGrid = nil then
		exit;
	WasDropped := IsWindowVisible(FDropGrid.Handle);
	CloseUp;
	DropDown(WasDropped);
	end;


procedure TDBFilterCombo.WMPaint(var Message: TWMPaint);
	var
	PS: TPaintStruct;
	ARect: TRect;
	S: array[0..255] of Char;
	Focused: Boolean;
	DC: HDC;
	begin
	Focused := GetFocus = Handle;
	if not ReadOnly then
		begin
		inherited;
		exit;
		end;
	if FCanvas = nil then
		begin
		FCanvas := TControlCanvas.Create;
		FCanvas.Control := Self;
		end;
	DC := Message.DC;
	if DC = 0 then
		DC := BeginPaint(Handle, PS);
	FCanvas.Handle := DC;
	try
		FCanvas.Font := Font;
		with FCanvas do
			begin
			ARect := ClientRect;
{$IFNDEF WIN32}
			Brush.Color := clWindowFrame;
			FrameRect(ARect);	 { draw the border }
			Inc(ARect.Left, 2);
			Inc(ARect.Top, 2);
{$ELSE}
			Inc(ARect.Left);
			Inc(ARect.Top);
{$ENDIF}
			ARect.Right := FButtonHolder.Left - 1;
			ARect.Bottom := ARect.Top + TextHeight(Text) + 1;
			Brush.Style := bsSolid;
			Brush.Color := Color;
			FillRect(ARect);
			StrPCopy(S, Text);
			if Focused then
				begin
				Brush.Color := clHighlight;
				Font.Color := clHighlightText;
				end;
			ExtTextOut(FCanvas.Handle, ARect.Left, ARect.Top, ETO_OPAQUE or ETO_CLIPPED, @ARect, S, StrLen(S), nil);
			if Focused then
				DrawFocusRect(ARect);
			end;
	finally
		FCanvas.Handle := 0;
		if Message.DC = 0 then
			EndPaint(Handle, PS);
		end;
	end;


procedure TDBFilterCombo.DoExit;
	begin
	if FDataBaseOK and (FCurrentText <> Text) then
		DataUpdate(self);
	inherited DoExit;
	end;


procedure TDBFilterCombo.Change;
	begin
	if FDataBaseOK then
		FDataLink.Modified;
	inherited Change;
	end;


function TDBFilterCombo.FitsFilter: boolean;
	{// Tight Filter Comparison ///////////////////////////////////////////////}
	var
	FldText, FltMax, FltMin: string;
	function CmpTxt(s1, s2: string): integer;
		var
		j: integer;
		begin
		Result := Length(s1) - Length(s2);
		if Result <> 0 then
			exit;
		for j := 1 to Length(s1) do
			begin
			Result := Ord(s1[j]) - Ord(s2[j]);
			if Result <> 0 then
				exit;
			end;
		end;

	begin
	if FNoFilter then
		begin
		Result := True;
		exit;
		end;
	Result := False;
	if FLookupUseCase then
		begin
		FldText := FFilterFld.Text;
		FltMax := FFilterMax;
		FltMin := FFilterMin;
		end
	else
		begin
		FldText := UpperCase(FFilterFld.Text);
		FltMax := UpperCase(FFilterMax);
		FltMin := UpperCase(FFilterMin);
		end;
	if FFilterRelaxed then
		begin
		if FFilterExclusive then
			begin
			if (FldText > FltMin) and
			   (FldText < FltMax) then
				Result := True;
			end
		else
			begin
			if (FldText >= FltMin) and
			   (FldText <= FltMax) then
				Result := True;
			end;
		end
	else
		begin
		if FFilterExclusive then
			begin
			if (CmpTxt(FldText, FltMin) > 0) and
			   (CmpTxt(FldText, FltMax) < 0) then
				Result := True;
			end
		else
			begin
			if (CmpTxt(FldText, FltMin) >= 0) and
			   (CmpTxt(FldText, FltMax) <= 0) then
				Result := True;
			end;
		end;
	end;


procedure TDBFilterCombo.WMPaste(var Message: TMessage);
	begin
	if not ReadOnly then
		begin
		FDataLink.Edit;
		FDataLink.Modified;
		inherited;
		end;
	end;


procedure TDBFilterCombo.WMCut(var Message: TMessage);
	begin
	if not ReadOnly then
		begin
		FDataLink.Edit;
		FDataLink.Modified;
		inherited;
		end;
	end;


procedure TDBFilterCombo.WMListClick(var Message: TMessage);
	var
	OldLookupItem: string;
	begin
	SetFocus;
	if FDropGrid = nil then
		exit;
	if FDataBaseOK then
		if Message.WParam = 1 then
			begin
			if Assigned(FOnListClick) then
				begin
				FCallingEvent := True;
				FOnListClick(self, FDropGrid, FDropGrid.Row);
				FCallingEvent := False;
				end;
			OldLookupItem := FLookupItem;
			LookupGridRow(FDropGrid.Row, OldLookupItem);
			SelectAll;
			end
		else
			FDataLink.Reset;
	CloseUp;
	end;


(* Windows bug with EM_SETRECTNP
procedure TDBFilterCombo.SetEditRect;
	var
	Loc: TRect;
	begin
	Loc.Bottom := ClientHeight + 1;  {+1 is workaround for windows paint bug}
	Loc.Right := FButton.Left - 2;
	Loc.Top := 0;
	Loc.Left := 0;
	Perform(EM_SETRECTNP, 0, LongInt(@Loc));
	end;
*)


procedure TDBFilterCombo.LookupGridRow(Row: integer; OldLookup: string);
	begin
	FDataLink.Edit;
	FDropGrid.Row := Row;
	if Text <> FDropGrid.Cells[0, FDropGrid.Row] then
		Text := FDropGrid.Cells[0, FDropGrid.Row];
	FLookupItem := FDropGrid.FLookupItems[FDropGrid.Row];
	FLookupSrc.First;
	FLookupSrc.MoveBy(integer(FDropGrid.FLookupCurs[FDropGrid.Row]));
	FDataLink.Field.Text := FLookupItem;
	FDataLink.Modified;
	if Assigned(FOnLookupChanged) and (OldLookup <> FLookupItem) then
		begin
		FCallingEvent := True;
		FOnLookupChanged(self, FDropGrid, FDropGrid.Row);
		FCallingEvent := False;
		end;
	end;


procedure TDBFilterCombo.WMKeyDown(var Message: TWMKey);
	var
	Key: word;
	OldLookupItem: string;
	begin
	Key := Message.CharCode;
	if Key = VK_TAB then
		begin
		inherited;
		exit;
		end;
	OldLookupItem := FLookupItem;
	if Key in [VK_UP, VK_DOWN] then
		begin
		if (FDropGrid <> nil)
			and IsWindowVisible(FDropGrid.Handle) then
			begin
			if (Key = VK_DOWN) and (FDropGrid.Row < (FDropGrid.RowCount - 1)) then
				LookupGridRow(FDropGrid.Row + 1, OldLookupItem);
			if (Key = VK_UP) and (FDropGrid.Row > FColNames) then
				LookupGridRow(FDropGrid.Row - 1, OldLookupItem);
			SelectAll;
			end
		else
			DropDown(True);
		exit;
		end;
	if Key = VK_ESCAPE then
		begin
		FDataLink.Reset;
		CloseUp;
		exit;
		end;
	if (FDropGrid <> nil)
		and IsWindowVisible(FDropGrid.Handle) then
		begin
		case Key of
			VK_RETURN:
				begin
				PostMessage(Handle, WM_LISTCLICK, 1, 0);
				CloseUp;
				end;
			VK_HOME:
				if FDropGrid.Row <> FColNames then
					LookupGridRow(FColNames, OldLookupItem);
			VK_END:
				if FDropGrid.Row <> (FDropGrid.RowCount - 1) then
					LookupGridRow(FDropGrid.RowCount - 1, OldLookupItem);
			end;
		if Key in [VK_HOME, VK_END, VK_RETURN] then
			exit;
		end;
	if (Key = VK_RETURN) then	{kill this because edit control is multiline}
		exit;
	inherited;
	end;


procedure TDBFilterCombo.WMKeyPress(var Message: TWMKey);
	var
	i, IncLen: integer;
	Key: word;
	TempText, MatchText, OldLookupItem: string;
	Matched: boolean;
	begin
	Key := Message.CharCode;
	if not (char(Key) in [' '..'~']) then
		begin
		if (Key <> VK_RETURN) then	{kill this because edit control is multiline}
			inherited;
		exit;
		end;
	if FIncSearch
		and (FAutoDrop = adOnChange)
		and (FDropGrid <> nil)
		and not IsWindowVisible(FDropGrid.Handle) then
		DropDown(True);
	case FCase of
		ccUpperCase:
			Key := word(UpperCase(char(Key))[1]);
		ccLowerCase:
			Key := word(LowerCase(char(Key))[1]);
		end;
	if (FStyle = csDropDown) and (not FIncSearch) then
		begin
		inherited;
		exit;
		end;
	TempText := Copy(Text, 1, SelStart) + char(Key);
	MatchText := TempText + Copy(Text, Length(TempText) + 1, Length(Text));
	IncLen := Length(TempText);
	if (FDropGrid <> nil)
		and FIncSearch
		and FDataBaseOK then
		begin
		Matched := False;
		for i := 0 to FDropGrid.RowCount - 1 do
			if ((FIncUseCase and (AnsiCompareStr(MatchText, FDropGrid.Cells[0, i]) = 0))
				or ((not FIncUseCase) and (AnsiCompareText(MatchText, FDropGrid.Cells[0, i]) = 0)))
				and (FLookupItem = FDropGrid.FLookupItems[i]) then
				begin
				Matched := True;
				break;
				end;
		if not Matched then
			begin
			OldLookupItem := FLookupItem;
			for i := 0 to FDropGrid.RowCount - 1 do
				begin
				if (FIncUseCase and (AnsiCompareStr(TempText, Copy(FDropGrid.Cells[0, i], 1, IncLen)) = 0))
					or ((not FIncUseCase) and (AnsiCompareText(TempText, Copy(FDropGrid.Cells[0, i], 1, IncLen)) = 0)) then
					begin
					if (TempText <> FDropGrid.Cells[0, i]) then
						LookupGridRow(i, OldLookupItem)
					else
						begin
						FDropGrid.Row := i;
						Text := FDropGrid.Cells[0, i];
						end;
					Matched := True;
					break;
					end;
				end;
			end;
		if not Matched then
			if FStyle = csDropDownList then
				begin
				if FIncErrorBeep then
					MessageBeep(0);
				Dec(IncLen);
				end
			else
				begin
				Text := TempText;
				FLookupItem := #1;
				end;
		SelStart := IncLen;
		if Length(Text) - IncLen > 0 then
			SelLength := Length(Text) - IncLen
		else
			SelLength := 0;
		end
	else
		inherited;
	end;


procedure TDBFilterCombo.WMKeyUp(var Message: TWMKey);
	var
	Key: word;
	begin
	Key := Message.CharCode;
	if (Key = VK_RETURN) then	{kill this because edit control is multiline}
		exit;
	inherited;
	end;


procedure TDBFilterCombo.Notification(AComponent: TComponent; Operation: TOperation);
	begin
	inherited Notification(AComponent, Operation);
	if (Operation = opRemove) and (FDataLink <> nil) then
		begin
		if (AComponent = LookupSource) then
			begin
			LookupSource := nil;
			FLookupSource := nil;
			AssignFields;
			end;
		if (AComponent = DataSource) then
			begin
			DataSource := nil;
			FDataLink.DataSource := nil;
			AssignFields;
			end
		end;
	end;


procedure TDBFilterCombo.DataChange(Sender: TObject);
	var
	OldIndexName, OldIndexFieldNames: string;
	begin
	if not FDataBaseOK then
		AssignFields;
	if FUpdating
		or (not FDataBaseOK)
		or (not (FLookupSrc is TTable))
		or (FDataLink.Field = nil)
		or (csLoading in ComponentState) then
		exit;
	with TTable(FLookupSrc) do
		begin
		OldIndexName := IndexName;
		OldIndexFieldNames := IndexFieldNames;
		IndexFieldNames := FLookupField;
		SetKey;
		FLookupFld.Text := FDataLink.Field.Text;
		if GoToKey then
			begin
			Text := TField(FDisplayList.Objects[0]).Text;
			FLookupItem := FLookupFld.Text;
			end
		else
			begin
			Text := '';
			FLookupItem := '';
			end;
		FCurrentText := Text;
		IndexName := OldIndexName;
		IndexFieldNames := OldIndexFieldNames;
		end;
	end;


procedure TDBFilterCombo.DataUpdate(Sender: TObject);
	var
	i: integer;
	Cancelled: boolean;
	begin
	if not FDataBaseOK then
		AssignFields;
	if FUpdating
		or FCallingEvent
		or (not FDataBaseOK)
		or (FStyle = csDropDownList)
		or (Text = '')
		or (csDesigning in ComponentState) then
		exit;
	FUpdating := True;
	FLookupSrc.Append;
	FCurrentText := Text;
	TField(FDisplayList.Objects[0]).Text := Text;
	if FFilterFld <> nil then
		FFilterFld.Text := FFilterMin;
	Cancelled := False;
	if not LookupExists then
		begin
		FLookupSrc.Insert;
		TField(FDisplayList.Objects[0]).Text := Text;
		for i := 1 to FDisplayList.Count - 1 do
			TField(FDisplayList.Objects[i]).Text := '';
		if FFilterFld <> nil then
			FFilterFld.Text := FFilterMin;
		if Assigned(FOnNewLookupRecord) then
			begin
			FCallingEvent := True;
			FOnNewLookupRecord(Self, Cancelled);
			FCallingEvent := False;
			end;
		if not Cancelled then
			begin
			try
				FLookupSrc.Post;
				DataLinkUpdate;
				FLookupItem := FLookupFld.Text;
			except
				end;
			end;
		end;
	FLookupSrc.Cancel;
	if LookupExists then
		DataLinkUpdate;
	FUpdating := False;
	end;


function TDBFilterCombo.LookupExists: boolean;
	begin
	FLookupSrc.Cancel;
	FLookupSrc.First;
	Result := True;
	while not FLookupSrc.EOF do
		begin
		if (FNoFilter or FitsFilter)
			and ((TField(FDisplayList.Objects[0]).Text = FCurrentText)
			or ((not FLookupUseCase) and (UpperCase(TField(FDisplayList.Objects[0]).Text) = UpperCase(FCurrentText)))) then
			exit;
		FLookupSrc.Next;
		end;
	Result := False;
	end;


procedure TDBFilterCombo.DataLinkUpdate;
	begin
	if FDataLink.Active then
		begin
		FDataLink.Edit;
		FLookupItem := FLookupFld.Text;
		if Text = '' then
			FLookupItem := '';
		FDataLink.Field.Text := FLookupItem;
		FDataLink.Modified;
		FDataLink.UpdateRecord;
		end;
	end;


{// Property Interface ////////////////////////////////////////////////////////}
function TDBFilterCombo.GetDataSource: TDataSource;
	begin
	Result := FDataLink.DataSource;
	end;


function TDBFilterCombo.GetDataField: string;
	begin
	Result := FDataLink.FieldName;
	end;


procedure TDBFilterCombo.SetDataSource(Src: TDataSource);
	begin
	if FDataLink.DataSource <> Src then
		begin
		FDataLink.DataSource := Src;
		AssignFields;
		end;
	end;


procedure TDBFilterCombo.SetLookupSource(Src: TDataSource);
	begin
	if FLookupSource <> Src then
		begin
		FLookupSource := Src;
		AssignFields;
		end;
	end;


procedure TDBFilterCombo.SetDataField(Str: string);
	begin
	try
		if FDataLink.FieldName <> Str then
			begin
			if (FDataLink.DataSource <> nil) and (FDataLink.DataSource.DataSet <> nil) then
				FDataLink.DataSource.DataSet.FieldByName(Str);
			FDataLink.FieldName := Str;
			AssignFields;
			end;
	except
{$IFDEF VER100}
		raise EInvalidOperation.Create(Format(SFieldNotFound, [Str]))
{$ELSE}
{$IFDEF VER120}
		raise EInvalidOperation.Create(Format(SFieldNotFound, [Str]))
{$ELSE}
		raise EInvalidOperation.Create(FmtLoadStr(SFieldNotFound, [Str]))
{$ENDIF}
{$ENDIF}
		end;
	end;


procedure TDBFilterCombo.SetLookupField(Str: string);
	begin
	try
		if FLookupField <> Str then
			begin
			if (FLookupSource <> nil) and (FLookupSource.DataSet <> nil) then
				FLookupSource.DataSet.FieldByName(Str);
			FLookupField := Str;
			AssignFields;
			end;
	except
{$IFDEF VER100}
		raise EInvalidOperation.Create(Format(SFieldNotFound, [Str]))
{$ELSE}
{$IFDEF VER120}
		raise EInvalidOperation.Create(Format(SFieldNotFound, [Str]))
{$ELSE}
		raise EInvalidOperation.Create(FmtLoadStr(SFieldNotFound, [Str]))
{$ENDIF}
{$ENDIF}
		end;
	end;


procedure TDBFilterCombo.SetDisplayField(Str: string);
	var
	i: integer;
	s, t: string;
	begin
	try
		if FDisplayField <> Str then
			begin
			FDisplayList.Clear;
			s := Str;
			while length(s) > 0 {Pos(';', s) > 0} do
				begin
				i := Pos(';', s);
				if i > 1 then
					begin
					t := Copy(s, 1, i - 1);
					Delete(s, 1, i);
					end
				else
					begin
					t := s;
					s := '';
					end;
				if length(t) > 0 then
					FDisplayList.Add(t);
				end;
			if (FLookupSource <> nil) and (FLookupSource.DataSet <> nil) then
				for i := 0 to FDisplayList.Count - 1 do
					begin
					s := FDisplayList[i];
					FLookupSource.DataSet.FieldByName(s);
					end;
			FDisplayField := Str;
			AssignFields;
			end;
	except
		FDisplayList.Clear;
{$IFDEF VER100}
		raise EInvalidOperation.Create(Format(SFieldNotFound, [s]))
{$ELSE}
{$IFDEF VER120}
		raise EInvalidOperation.Create(Format(SFieldNotFound, [s]))
{$ELSE}
		raise EInvalidOperation.Create(FmtLoadStr(SFieldNotFound, [s]))
{$ENDIF}
{$ENDIF}
		end;
	end;


procedure TDBFilterCombo.SetFilterField(Str: string);
	begin
	try
		if FFilterField <> Str then
			begin
			if (FLookupSource <> nil) and (FLookupSource.DataSet <> nil) then
				FLookupSource.DataSet.FieldByName(Str);
			FFilterField := Str;
			AssignFields;
			end;
	except
{$IFDEF VER100}
		raise EInvalidOperation.Create(Format(SFieldNotFound, [Str]))
{$ELSE}
{$IFDEF VER120}
		raise EInvalidOperation.Create(Format(SFieldNotFound, [Str]))
{$ELSE}
		raise EInvalidOperation.Create(FmtLoadStr(SFieldNotFound, [Str]))
{$ENDIF}
{$ENDIF}
		end;
	end;


procedure TDBFilterCombo.SetIncSearch(Value: boolean);
	begin
	FIncSearch := Value;
	ReadOnly := (FStyle = csDropDownList) and (not FIncSearch);
	end;


procedure TDBFilterCombo.SetStyle(Value: TDBFilterComboStyle);
	begin
	FStyle := Value;
	ReadOnly := (FStyle = csDropDownList) and (not FIncSearch);
	end;


procedure TDBFilterCombo.SetDropFont(Value: TFont);
	begin
	FParentDropFont := False;
	FDropFont.Assign(Value);
	end;


procedure TDBFilterCombo.SetDropLineWidth(Value: integer);
	begin
	if FDropLineWidth = Value then
		exit;
	FDropLineWidth := Value;
	end;


procedure TDBFilterCombo.SetDropWidth(Value: integer);
	begin
	if FDropWid = Value then
		exit;
	FDropWid := Value;
	end;


{// Registration //////////////////////////////////////////////////////////////}
procedure Register;
	begin
	RegisterComponents('Data Controls', [TDBFilterCombo]);
	end;


{// Initialization ////////////////////////////////////////////////////////////}
initialization
	begin
{$IFDEF WIN32}
	ButtonSize := GetSystemMetrics(SM_CXVSCROLL);
{$ELSE}
	ButtonSize := 17;
{$ENDIF}
	Tricky := 'DBFilterCombo Copyright  1995-1999 Steve Flynn, All Rights Reserved.'#0#0#0#0;
	CopyrightNotice := StrPas(@Tricky[1]) + #13#13;
{$IFDEF WIN32}
	CopyrightNotice := CopyrightNotice + ' 32Bit';
{$ELSE}
	CopyrightNotice := CopyrightNotice + ' 16Bit';
{$ENDIF}
	CopyrightNotice := CopyrightNotice + ' Version 4.0';
	CopyrightNotice := CopyrightNotice + #13#13'By using this Software you agree to be bound to the terms';
	CopyrightNotice := CopyrightNotice + #13'of the License Agreement provided with this Software.';
	end;

end.

